diff --git a/src/common/libmmgtypes.h b/src/common/libmmgtypes.h
index 65e1f1c..c34677b 100644
--- a/src/common/libmmgtypes.h
+++ b/src/common/libmmgtypes.h
@@ -196,6 +196,7 @@ typedef struct {
   double   c[3]; /*!< Coordinates of point */
   double   n[3]; /*!< Normal or Tangent for mmgs and Tangent (if needed) for mmg3d */
   int      ref; /*!< Reference of point */
+  int      init_index; /*!< index of init point to keep lonely edges */
   int      xp; /*!< Surface point number */
   int      tmp; /*!< Index of point in the saved mesh (we don't count
                   the unused points)*/
diff --git a/src/mmg3d/analys_3d.c b/src/mmg3d/analys_3d.c
index 13e6c0e..acbe99e 100644
--- a/src/mmg3d/analys_3d.c
+++ b/src/mmg3d/analys_3d.c
@@ -647,6 +647,7 @@ static int _MMG3D_nmgeom(MMG5_pMesh mesh){
 int _MMG3D_analys(MMG5_pMesh mesh) {
   _MMG5_Hash hash;
 
+
   /**--- stage 1: data structures for surface */
   if ( abs(mesh->info.imprim) > 3 )
     fprintf(stdout,"  ** SURFACE ANALYSIS\n");
diff --git a/src/mmg3d/hash_3d.c b/src/mmg3d/hash_3d.c
index d72c046..8577678 100644
--- a/src/mmg3d/hash_3d.c
+++ b/src/mmg3d/hash_3d.c
@@ -1010,6 +1010,7 @@ int _MMG5_hGeom(MMG5_pMesh mesh) {
   MMG5_pTria   pt;
   MMG5_pEdge   pa;
   _MMG5_Hash   hash;
+  MMG5_HGeom   foundEdges;
   int         *adja,k,kk,edg,ier;
   int16_t      tag;
   char         i,i1,i2;
@@ -1039,6 +1040,10 @@ int _MMG5_hGeom(MMG5_pMesh mesh) {
     }
 
     /* now check triangles */
+
+    if ( !_MMG5_hNew(mesh,&foundEdges,mesh->na,3*mesh->namax) )
+      return 0;
+
     for (k=1; k<=mesh->nt; k++) {
       pt = &mesh->tria[k];
       for (i=0; i<3; i++) {
@@ -1051,6 +1056,10 @@ int _MMG5_hGeom(MMG5_pMesh mesh) {
         _MMG5_hGet(&mesh->htab,pt->v[i1],pt->v[i2],&edg,&tag);
         pt->edg[i]  = edg;
 
+        /* Hash the edges found inside trias to be able to delete it */
+        if ( !_MMG5_hEdge(mesh,&foundEdges,pt->v[i1],pt->v[i2],0,0) )
+          return 0;
+
         /* If we use the nosurf option and the edge is required, we don't want
          * to detect it as an edge whose tag has been modified for the option */
         if ( mesh->info.nosurf && (tag & MG_REQ) )
@@ -1061,8 +1070,28 @@ int _MMG5_hGeom(MMG5_pMesh mesh) {
         _MMG5_hTag(&mesh->htab,pt->v[i1],pt->v[i2],edg,pt->tag[i]);
       }
     }
-    _MMG5_DEL_MEM(mesh,mesh->edge,(mesh->na+1)*sizeof(MMG5_Edge));
-    mesh->na   = 0;
+
+    /* Keep only the lonely edges and pack it */
+    int na  = 0;
+    int nbl = 1;
+    for (k=1; k<=mesh->na; k++) {
+      pa = &mesh->edge[k];
+
+      if ( _MMG5_hGet(&foundEdges,pa->a,pa->b,&edg,&tag) )
+        continue;
+
+      ++na;
+      if ( k!=nbl ) {
+        memcpy(&mesh->edge[nbl],pa,sizeof(MMG5_Edge));
+      }
+      ++nbl;
+    }
+    (mesh)->memCur -= (mesh->na-na)*sizeof(MMG5_Edge);
+    mesh->na = na;
+    _MMG5_DEL_MEM(mesh,foundEdges.geom,(foundEdges.max+1)*sizeof(MMG5_hgeom));
+
+    /* _MMG5_DEL_MEM(mesh,mesh->edge,(mesh->na+1)*sizeof(MMG5_Edge)); */
+    /* mesh->na   = 0; */
   }
   /* else, infer special edges from information carried by triangles */
   else {
diff --git a/src/mmg3d/inout_3d.c b/src/mmg3d/inout_3d.c
index d3d82d9..2d6957e 100644
--- a/src/mmg3d/inout_3d.c
+++ b/src/mmg3d/inout_3d.c
@@ -662,6 +662,13 @@ int MMG3D_loadMesh(MMG5_pMesh mesh,const char *filename) {
         fread(&pa->ref,sw,1,inm);
         if(iswp) pa->ref=_MMG5_swapbin(pa->ref);
       }
+
+      mesh->point[pa->a].tag &= ~MG_NUL;
+      mesh->point[pa->b].tag &= ~MG_NUL;
+      mesh->point[pa->a].init_index = pa->a;
+      mesh->point[pa->b].init_index = pa->b;
+
+
       pa->tag |= MG_REF;
       if ( mesh->info.iso ) {
         if( abs(pa->ref) != MG_ISO ) {
diff --git a/src/mmg3d/libmmg3d.c b/src/mmg3d/libmmg3d.c
index 7fb0966..e69d9f8 100644
--- a/src/mmg3d/libmmg3d.c
+++ b/src/mmg3d/libmmg3d.c
@@ -234,10 +234,36 @@ int _MMG3D_bdryBuild(MMG5_pMesh mesh) {
   if ( mesh->htab.geom )
     _MMG5_DEL_MEM(mesh,mesh->htab.geom,(mesh->htab.max+1)*sizeof(MMG5_hgeom));
 
-  mesh->na = nr = 0;
+  //mesh->na = nr = 0;
+  nr = 0;
+
   /* in the worst case (all edges are marked), we will have around 1 edge per *
    * triangle (we count edges only one time) */
   if ( _MMG5_hNew(mesh,&mesh->htab,mesh->nt,3*(mesh->nt)) ) {
+
+    /* Hash the lonely edges first */
+    MMG5_pEdge pa;
+    int        a, b;
+    for (k=1; k<=mesh->na; k++) {
+      pa = &mesh->edge[k];
+
+      int found = 0;
+      for ( int kk=1; kk<=mesh->np; ++kk ) {
+        if ( mesh->point[kk].init_index == pa->a ) {
+          a = mesh->point[kk].tmp;
+          ++found;
+        }
+        else if ( mesh->point[kk].init_index == pa->b ) {
+          b = mesh->point[kk].tmp;
+          ++found;
+        }
+        if ( found > 1 ) break;
+      }
+      if ( found )
+        _MMG5_hEdge(mesh,&mesh->htab,a,b,pa->ref,pa->tag);
+    }
+    _MMG5_DEL_MEM(mesh,mesh->edge,(mesh->na+1)*sizeof(MMG5_Edge));
+
     for (k=1; k<=mesh->ne; k++) {
       pt   = &mesh->tetra[k];
       if ( MG_EOK(pt) &&  pt->xt ) {
@@ -253,6 +279,7 @@ int _MMG3D_bdryBuild(MMG5_pMesh mesh) {
     }
 
     /* edges + ridges + required edges */
+    mesh->na = 0;
     for (k=0; k<=mesh->htab.max; k++) {
       ph = &mesh->htab.geom[k];
       if ( !(ph->a) )  continue;
